﻿namespace HTMLConverter
{
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.Diagnostics;
    using System.Xml;

    internal static class HtmlCssParser
    {
        private static string[] _blocks = new string[] { "block", "inline", "list-item", "none" };
        private static readonly string[] _borderStyles = new string[] { "none", "dotted", "dashed", "solid", "double", "groove", "ridge", "inset", "outset" };
        private static readonly string[] _clears = new string[] { "none", "left", "right", "both" };
        private static readonly string[] _colors = new string[] { 
            "aliceblue", "antiquewhite", "aqua", "aquamarine", "azure", "beige", "bisque", "black", "blanchedalmond", "blue", "blueviolet", "brown", "burlywood", "cadetblue", "chartreuse", "chocolate", 
            "coral", "cornflowerblue", "cornsilk", "crimson", "cyan", "darkblue", "darkcyan", "darkgoldenrod", "darkgray", "darkgreen", "darkkhaki", "darkmagenta", "darkolivegreen", "darkorange", "darkorchid", "darkred", 
            "darksalmon", "darkseagreen", "darkslateblue", "darkslategray", "darkturquoise", "darkviolet", "deeppink", "deepskyblue", "dimgray", "dodgerblue", "firebrick", "floralwhite", "forestgreen", "fuchsia", "gainsboro", "ghostwhite", 
            "gold", "goldenrod", "gray", "green", "greenyellow", "honeydew", "hotpink", "indianred", "indigo", "ivory", "khaki", "lavender", "lavenderblush", "lawngreen", "lemonchiffon", "lightblue", 
            "lightcoral", "lightcyan", "lightgoldenrodyellow", "lightgreen", "lightgrey", "lightpink", "lightsalmon", "lightseagreen", "lightskyblue", "lightslategray", "lightsteelblue", "lightyellow", "lime", "limegreen", "linen", "magenta", 
            "maroon", "mediumaquamarine", "mediumblue", "mediumorchid", "mediumpurple", "mediumseagreen", "mediumslateblue", "mediumspringgreen", "mediumturquoise", "mediumvioletred", "midnightblue", "mintcream", "mistyrose", "moccasin", "navajowhite", "navy", 
            "oldlace", "olive", "olivedrab", "orange", "orangered", "orchid", "palegoldenrod", "palegreen", "paleturquoise", "palevioletred", "papayawhip", "peachpuff", "peru", "pink", "plum", "powderblue", 
            "purple", "red", "rosybrown", "royalblue", "saddlebrown", "salmon", "sandybrown", "seagreen", "seashell", "sienna", "silver", "skyblue", "slateblue", "slategray", "snow", "springgreen", 
            "steelblue", "tan", "teal", "thistle", "tomato", "turquoise", "violet", "wheat", "white", "whitesmoke", "yellow", "yellowgreen"
         };
        private static readonly string[] _floats = new string[] { "left", "right", "none" };
        private static readonly string[] _fontAbsoluteSizes = new string[] { "xx-small", "x-small", "small", "medium", "large", "x-large", "xx-large" };
        private static readonly string[] _fontGenericFamilies = new string[] { "serif", "sans-serif", "monospace", "cursive", "fantasy" };
        private static readonly string[] _fontRelativeSizes = new string[] { "larger", "smaller" };
        private static readonly string[] _fontSizeUnits = new string[] { "px", "mm", "cm", "in", "pt", "pc", "em", "ex", "%" };
        private static readonly string[] _fontStyles = new string[] { "normal", "italic", "oblique" };
        private static readonly string[] _fontVariants = new string[] { "normal", "small-caps" };
        private static readonly string[] _fontWeights = new string[] { "normal", "bold", "bolder", "lighter", "100", "200", "300", "400", "500", "600", "700", "800", "900" };
        private static readonly string[] _listStylePositions = new string[] { "inside", "outside" };
        private static readonly string[] _listStyleTypes = new string[] { "disc", "circle", "square", "decimal", "lower-roman", "upper-roman", "lower-alpha", "upper-alpha", "none" };
        private static readonly string[] _systemColors = new string[] { 
            "activeborder", "activecaption", "appworkspace", "background", "buttonface", "buttonhighlight", "buttonshadow", "buttontext", "captiontext", "graytext", "highlight", "highlighttext", "inactiveborder", "inactivecaption", "inactivecaptiontext", "infobackground", 
            "infotext", "menu", "menutext", "scrollbar", "threeddarkshadow", "threedface", "threedhighlight", "threedlightshadow", "threedshadow", "window", "windowframe", "windowtext"
         };
        private static readonly string[] _textAligns = new string[] { "left", "right", "center", "justify" };
        private static readonly string[] _textDecorations = new string[] { "none", "underline", "overline", "line-through", "blink" };
        private static readonly string[] _textTransforms = new string[] { "none", "capitalize", "uppercase", "lowercase" };
        private static readonly string[] _verticalAligns = new string[] { "baseline", "sub", "super", "top", "text-top", "middle", "bottom", "text-bottom" };

        internal static void GetElementPropertiesFromCssAttributes(XmlElement htmlElement, string elementName, CssStylesheet stylesheet, Hashtable localProperties, List<XmlElement> sourceContext)
        {
            string style = stylesheet.GetStyle(elementName, sourceContext);
            string attribute = HtmlToXamlConverter.GetAttribute(htmlElement, "style");
            string str3 = (style != null) ? style : null;
            if (attribute != null)
            {
                str3 = (str3 == null) ? attribute : (str3 + ";" + attribute);
            }
            if (str3 != null)
            {
                string[] strArray = str3.Split(new char[] { ';' });
                for (int i = 0; i < strArray.Length; i++)
                {
                    string[] strArray2 = strArray[i].Split(new char[] { ':' });
                    if (strArray2.Length == 2)
                    {
                        string propertyName = strArray2[0].Trim().ToLower();
                        string styleValue = HtmlToXamlConverter.UnQuote(strArray2[1].Trim()).ToLower();
                        int nextIndex = 0;
                        switch (propertyName)
                        {
                            case "font":
                                ParseCssFont(styleValue, localProperties);
                                break;

                            case "font-family":
                                ParseCssFontFamily(styleValue, ref nextIndex, localProperties);
                                break;

                            case "font-size":
                                ParseCssSize(styleValue, ref nextIndex, localProperties, "font-size", true);
                                break;

                            case "font-style":
                                ParseCssFontStyle(styleValue, ref nextIndex, localProperties);
                                break;

                            case "font-weight":
                                ParseCssFontWeight(styleValue, ref nextIndex, localProperties);
                                break;

                            case "font-variant":
                                ParseCssFontVariant(styleValue, ref nextIndex, localProperties);
                                break;

                            case "line-height":
                                ParseCssSize(styleValue, ref nextIndex, localProperties, "line-height", true);
                                break;

                            case "color":
                                ParseCssColor(styleValue, ref nextIndex, localProperties, "color");
                                break;

                            case "text-decoration":
                                ParseCssTextDecoration(styleValue, ref nextIndex, localProperties);
                                break;

                            case "text-transform":
                                ParseCssTextTransform(styleValue, ref nextIndex, localProperties);
                                break;

                            case "background-color":
                                ParseCssColor(styleValue, ref nextIndex, localProperties, "background-color");
                                break;

                            case "background":
                                ParseCssBackground(styleValue, ref nextIndex, localProperties);
                                break;

                            case "text-align":
                                ParseCssTextAlign(styleValue, ref nextIndex, localProperties);
                                break;

                            case "vertical-align":
                                ParseCssVerticalAlign(styleValue, ref nextIndex, localProperties);
                                break;

                            case "text-indent":
                                ParseCssSize(styleValue, ref nextIndex, localProperties, "text-indent", false);
                                break;

                            case "width":
                            case "height":
                                ParseCssSize(styleValue, ref nextIndex, localProperties, propertyName, true);
                                break;

                            case "margin":
                                ParseCssRectangleProperty(styleValue, ref nextIndex, localProperties, propertyName);
                                break;

                            case "margin-top":
                            case "margin-right":
                            case "margin-bottom":
                            case "margin-left":
                                ParseCssSize(styleValue, ref nextIndex, localProperties, propertyName, true);
                                break;

                            case "padding":
                                ParseCssRectangleProperty(styleValue, ref nextIndex, localProperties, propertyName);
                                break;

                            case "padding-top":
                            case "padding-right":
                            case "padding-bottom":
                            case "padding-left":
                                ParseCssSize(styleValue, ref nextIndex, localProperties, propertyName, true);
                                break;

                            case "border":
                                ParseCssBorder(styleValue, ref nextIndex, localProperties);
                                break;

                            case "border-style":
                            case "border-width":
                            case "border-color":
                                ParseCssRectangleProperty(styleValue, ref nextIndex, localProperties, propertyName);
                                break;

                            case "float":
                                ParseCssFloat(styleValue, ref nextIndex, localProperties);
                                break;

                            case "clear":
                                ParseCssClear(styleValue, ref nextIndex, localProperties);
                                break;
                        }
                    }
                }
            }
        }

        private static void ParseCssBackground(string styleValue, ref int nextIndex, Hashtable localValues)
        {
        }

        private static void ParseCssBorder(string styleValue, ref int nextIndex, Hashtable localProperties)
        {
            while ((ParseCssRectangleProperty(styleValue, ref nextIndex, localProperties, "border-width") || ParseCssRectangleProperty(styleValue, ref nextIndex, localProperties, "border-style")) || ParseCssRectangleProperty(styleValue, ref nextIndex, localProperties, "border-color"))
            {
            }
        }

        private static string ParseCssBorderStyle(string styleValue, ref int nextIndex)
        {
            return ParseWordEnumeration(_borderStyles, styleValue, ref nextIndex);
        }

        private static void ParseCssClear(string styleValue, ref int nextIndex, Hashtable localProperties)
        {
            ParseWordEnumeration(_clears, styleValue, ref nextIndex, localProperties, "clear");
        }

        private static string ParseCssColor(string styleValue, ref int nextIndex)
        {
            ParseWhiteSpace(styleValue, ref nextIndex);
            string str = null;
            if (nextIndex < styleValue.Length)
            {
                int startIndex = nextIndex;
                char c = styleValue[nextIndex];
                if (c == '#')
                {
                    nextIndex++;
                    while (nextIndex < styleValue.Length)
                    {
                        c = char.ToUpper(styleValue[nextIndex]);
                        if ((('0' > c) || (c > '9')) && (('A' > c) || (c > 'F')))
                        {
                            break;
                        }
                        nextIndex++;
                    }
                    if (nextIndex > (startIndex + 1))
                    {
                        str = styleValue.Substring(startIndex, nextIndex - startIndex);
                    }
                    return str;
                }
                if (styleValue.Substring(nextIndex, 3).ToLower() == "rbg")
                {
                    while ((nextIndex < styleValue.Length) && (styleValue[nextIndex] != ')'))
                    {
                        nextIndex++;
                    }
                    if (nextIndex < styleValue.Length)
                    {
                        nextIndex++;
                    }
                    return "gray";
                }
                if (char.IsLetter(c))
                {
                    str = ParseWordEnumeration(_colors, styleValue, ref nextIndex);
                    if (str != null)
                    {
                        return str;
                    }
                    str = ParseWordEnumeration(_systemColors, styleValue, ref nextIndex);
                    if (str != null)
                    {
                        str = "black";
                    }
                }
            }
            return str;
        }

        private static void ParseCssColor(string styleValue, ref int nextIndex, Hashtable localValues, string propertyName)
        {
            string str = ParseCssColor(styleValue, ref nextIndex);
            if (str != null)
            {
                localValues[propertyName] = str;
            }
        }

        private static void ParseCssFloat(string styleValue, ref int nextIndex, Hashtable localProperties)
        {
            ParseWordEnumeration(_floats, styleValue, ref nextIndex, localProperties, "float");
        }

        private static void ParseCssFont(string styleValue, Hashtable localProperties)
        {
            int nextIndex = 0;
            ParseCssFontStyle(styleValue, ref nextIndex, localProperties);
            ParseCssFontVariant(styleValue, ref nextIndex, localProperties);
            ParseCssFontWeight(styleValue, ref nextIndex, localProperties);
            ParseCssSize(styleValue, ref nextIndex, localProperties, "font-size", true);
            ParseWhiteSpace(styleValue, ref nextIndex);
            if ((nextIndex < styleValue.Length) && (styleValue[nextIndex] == '/'))
            {
                nextIndex++;
                ParseCssSize(styleValue, ref nextIndex, localProperties, "line-height", true);
            }
            ParseCssFontFamily(styleValue, ref nextIndex, localProperties);
        }

        private static void ParseCssFontFamily(string styleValue, ref int nextIndex, Hashtable localProperties)
        {
            string str = null;
            while (nextIndex < styleValue.Length)
            {
                string str2 = ParseWordEnumeration(_fontGenericFamilies, styleValue, ref nextIndex);
                if (str2 == null)
                {
                    int num;
                    if ((nextIndex < styleValue.Length) && ((styleValue[nextIndex] == '"') || (styleValue[nextIndex] == '\'')))
                    {
                        char ch = styleValue[nextIndex];
                        nextIndex++;
                        num = nextIndex;
                        while ((nextIndex < styleValue.Length) && (styleValue[nextIndex] != ch))
                        {
                            nextIndex++;
                        }
                        str2 = '"' + styleValue.Substring(num, nextIndex - num) + '"';
                    }
                    if (str2 == null)
                    {
                        num = nextIndex;
                        while (((nextIndex < styleValue.Length) && (styleValue[nextIndex] != ',')) && (styleValue[nextIndex] != ';'))
                        {
                            nextIndex++;
                        }
                        if (nextIndex > num)
                        {
                            str2 = styleValue.Substring(num, nextIndex - num).Trim();
                            if (str2.Length == 0)
                            {
                                str2 = null;
                            }
                        }
                    }
                }
                ParseWhiteSpace(styleValue, ref nextIndex);
                if ((nextIndex < styleValue.Length) && (styleValue[nextIndex] == ','))
                {
                    nextIndex++;
                }
                if (str2 == null)
                {
                    break;
                }
                if ((str == null) && (str2.Length > 0))
                {
                    if ((str2[0] == '"') || (str2[0] == '\''))
                    {
                        str2 = str2.Substring(1, str2.Length - 2);
                    }
                    str = str2;
                }
            }
            if (str != null)
            {
                localProperties["font-family"] = str;
            }
        }

        private static void ParseCssFontStyle(string styleValue, ref int nextIndex, Hashtable localProperties)
        {
            ParseWordEnumeration(_fontStyles, styleValue, ref nextIndex, localProperties, "font-style");
        }

        private static void ParseCssFontVariant(string styleValue, ref int nextIndex, Hashtable localProperties)
        {
            ParseWordEnumeration(_fontVariants, styleValue, ref nextIndex, localProperties, "font-variant");
        }

        private static void ParseCssFontWeight(string styleValue, ref int nextIndex, Hashtable localProperties)
        {
            ParseWordEnumeration(_fontWeights, styleValue, ref nextIndex, localProperties, "font-weight");
        }

        private static void ParseCssListStyle(string styleValue, Hashtable localProperties)
        {
            int nextIndex = 0;
            while (nextIndex < styleValue.Length)
            {
                string str = ParseCssListStyleType(styleValue, ref nextIndex);
                if (str != null)
                {
                    localProperties["list-style-type"] = str;
                }
                else
                {
                    string str2 = ParseCssListStylePosition(styleValue, ref nextIndex);
                    if (str2 != null)
                    {
                        localProperties["list-style-position"] = str2;
                    }
                    else
                    {
                        string str3 = ParseCssListStyleImage(styleValue, ref nextIndex);
                        if (str3 != null)
                        {
                            localProperties["list-style-image"] = str3;
                        }
                        else
                        {
                            break;
                        }
                    }
                }
            }
        }

        private static string ParseCssListStyleImage(string styleValue, ref int nextIndex)
        {
            return null;
        }

        private static string ParseCssListStylePosition(string styleValue, ref int nextIndex)
        {
            return ParseWordEnumeration(_listStylePositions, styleValue, ref nextIndex);
        }

        private static string ParseCssListStyleType(string styleValue, ref int nextIndex)
        {
            return ParseWordEnumeration(_listStyleTypes, styleValue, ref nextIndex);
        }

        private static bool ParseCssRectangleProperty(string styleValue, ref int nextIndex, Hashtable localProperties, string propertyName)
        {
            Debug.Assert((((propertyName == "margin") || (propertyName == "padding")) || ((propertyName == "border-width") || (propertyName == "border-style"))) || (propertyName == "border-color"));
            string str = (propertyName == "border-color") ? ParseCssColor(styleValue, ref nextIndex) : ((propertyName == "border-style") ? ParseCssBorderStyle(styleValue, ref nextIndex) : ParseCssSize(styleValue, ref nextIndex, true));
            if (str != null)
            {
                localProperties[propertyName + "-top"] = str;
                localProperties[propertyName + "-bottom"] = str;
                localProperties[propertyName + "-right"] = str;
                localProperties[propertyName + "-left"] = str;
                str = (propertyName == "border-color") ? ParseCssColor(styleValue, ref nextIndex) : ((propertyName == "border-style") ? ParseCssBorderStyle(styleValue, ref nextIndex) : ParseCssSize(styleValue, ref nextIndex, true));
                if (str != null)
                {
                    localProperties[propertyName + "-right"] = str;
                    localProperties[propertyName + "-left"] = str;
                    str = (propertyName == "border-color") ? ParseCssColor(styleValue, ref nextIndex) : ((propertyName == "border-style") ? ParseCssBorderStyle(styleValue, ref nextIndex) : ParseCssSize(styleValue, ref nextIndex, true));
                    if (str != null)
                    {
                        localProperties[propertyName + "-bottom"] = str;
                        str = (propertyName == "border-color") ? ParseCssColor(styleValue, ref nextIndex) : ((propertyName == "border-style") ? ParseCssBorderStyle(styleValue, ref nextIndex) : ParseCssSize(styleValue, ref nextIndex, true));
                        if (str != null)
                        {
                            localProperties[propertyName + "-left"] = str;
                        }
                    }
                }
                return true;
            }
            return false;
        }

        private static string ParseCssSize(string styleValue, ref int nextIndex, bool mustBeNonNegative)
        {
            ParseWhiteSpace(styleValue, ref nextIndex);
            int startIndex = nextIndex;
            if ((nextIndex < styleValue.Length) && (styleValue[nextIndex] == '-'))
            {
                nextIndex++;
            }
            if ((nextIndex < styleValue.Length) && char.IsDigit(styleValue[nextIndex]))
            {
                while ((nextIndex < styleValue.Length) && (char.IsDigit(styleValue[nextIndex]) || (styleValue[nextIndex] == '.')))
                {
                    nextIndex++;
                }
                string str = styleValue.Substring(startIndex, nextIndex - startIndex);
                string str2 = ParseWordEnumeration(_fontSizeUnits, styleValue, ref nextIndex);
                if (str2 == null)
                {
                    str2 = "px";
                }
                if (mustBeNonNegative && (styleValue[startIndex] == '-'))
                {
                    return "0";
                }
                return (str + str2);
            }
            return null;
        }

        private static void ParseCssSize(string styleValue, ref int nextIndex, Hashtable localValues, string propertyName, bool mustBeNonNegative)
        {
            string str = ParseCssSize(styleValue, ref nextIndex, mustBeNonNegative);
            if (str != null)
            {
                localValues[propertyName] = str;
            }
        }

        private static void ParseCssTextAlign(string styleValue, ref int nextIndex, Hashtable localProperties)
        {
            ParseWordEnumeration(_textAligns, styleValue, ref nextIndex, localProperties, "text-align");
        }

        private static void ParseCssTextDecoration(string styleValue, ref int nextIndex, Hashtable localProperties)
        {
            for (int i = 1; i < _textDecorations.Length; i++)
            {
                localProperties["text-decoration-" + _textDecorations[i]] = "false";
            }
            while (nextIndex < styleValue.Length)
            {
                string str = ParseWordEnumeration(_textDecorations, styleValue, ref nextIndex);
                if ((str == null) || (str == "none"))
                {
                    break;
                }
                localProperties["text-decoration-" + str] = "true";
            }
        }

        private static void ParseCssTextTransform(string styleValue, ref int nextIndex, Hashtable localProperties)
        {
            ParseWordEnumeration(_textTransforms, styleValue, ref nextIndex, localProperties, "text-transform");
        }

        private static void ParseCssVerticalAlign(string styleValue, ref int nextIndex, Hashtable localProperties)
        {
            ParseWordEnumeration(_verticalAligns, styleValue, ref nextIndex, localProperties, "vertical-align");
        }

        private static void ParseWhiteSpace(string styleValue, ref int nextIndex)
        {
            while ((nextIndex < styleValue.Length) && char.IsWhiteSpace(styleValue[nextIndex]))
            {
                nextIndex++;
            }
        }

        private static bool ParseWord(string word, string styleValue, ref int nextIndex)
        {
            ParseWhiteSpace(styleValue, ref nextIndex);
            for (int i = 0; i < word.Length; i++)
            {
                if (((nextIndex + i) >= styleValue.Length) || (word[i] != styleValue[nextIndex + i]))
                {
                    return false;
                }
            }
            if (((nextIndex + word.Length) < styleValue.Length) && char.IsLetterOrDigit(styleValue[nextIndex + word.Length]))
            {
                return false;
            }
            nextIndex += word.Length;
            return true;
        }

        private static string ParseWordEnumeration(string[] words, string styleValue, ref int nextIndex)
        {
            for (int i = 0; i < words.Length; i++)
            {
                if (ParseWord(words[i], styleValue, ref nextIndex))
                {
                    return words[i];
                }
            }
            return null;
        }

        private static void ParseWordEnumeration(string[] words, string styleValue, ref int nextIndex, Hashtable localProperties, string attributeName)
        {
            string str = ParseWordEnumeration(words, styleValue, ref nextIndex);
            if (str != null)
            {
                localProperties[attributeName] = str;
            }
        }
    }
}

