﻿namespace WpfRichText.Ex.XamlToHtmlParser
{
    using System;
    using System.Diagnostics;
    using System.IO;
    using System.Text;
    using System.Xml;

    internal static class HtmlFromXamlConverter
    {
        private static void AddComplexProperty(XmlTextReader xamlReader, StringBuilder inlineStyle)
        {
            Debug.Assert(xamlReader.NodeType == XmlNodeType.Element);
            if ((inlineStyle != null) && xamlReader.Name.EndsWith(".TextDecorations"))
            {
                inlineStyle.Append("text-decoration:underline;");
            }
            WriteElementContent(xamlReader, null, null);
        }

        internal static string ConvertXamlToHtml(string xamlString)
        {
            return ConvertXamlToHtml(xamlString, true);
        }

        internal static string ConvertXamlToHtml(string xamlString, bool asFlowDocument)
        {
            if (!asFlowDocument)
            {
                xamlString = "<FlowDocument>" + xamlString + "</FlowDocument>";
            }
            XmlTextReader xamlReader = new XmlTextReader(new StringReader(xamlString));
            StringBuilder sb = new StringBuilder(100);
            XmlTextWriter htmlWriter = new XmlTextWriter(new StringWriter(sb));
            if (!WriteFlowDocument(xamlReader, htmlWriter))
            {
                return "";
            }
            return sb.ToString();
        }

        private static string ParseXamlColor(string color)
        {
            if (color.StartsWith("#"))
            {
                color = "#" + color.Substring(3);
            }
            return color;
        }

        private static string ParseXamlThickness(string thickness)
        {
            string[] strArray = thickness.Split(new char[] { ',' });
            for (int i = 0; i < strArray.Length; i++)
            {
                double num2;
                if (double.TryParse(strArray[i], out num2))
                {
                    strArray[i] = Math.Ceiling(num2).ToString();
                }
                else
                {
                    strArray[i] = "1";
                }
            }
            switch (strArray.Length)
            {
                case 1:
                    return thickness;

                case 2:
                    return (strArray[1] + " " + strArray[0]);

                case 4:
                    return (strArray[1] + " " + strArray[2] + " " + strArray[3] + " " + strArray[0]);
            }
            return strArray[0];
        }

        private static bool ReadNextToken(XmlReader xamlReader)
        {
            while (xamlReader.Read())
            {
                Debug.Assert(xamlReader.ReadState == ReadState.Interactive, "Reader is expected to be in Interactive state (" + xamlReader.ReadState + ")");
                switch (xamlReader.NodeType)
                {
                    case XmlNodeType.None:
                    case XmlNodeType.Element:
                    case XmlNodeType.Text:
                    case XmlNodeType.CDATA:
                    case XmlNodeType.SignificantWhitespace:
                    case XmlNodeType.EndElement:
                        return true;

                    case XmlNodeType.Comment:
                        return true;

                    case XmlNodeType.Whitespace:
                        if (xamlReader.XmlSpace != XmlSpace.Preserve)
                        {
                            break;
                        }
                        return true;
                }
            }
            return false;
        }

        private static void WriteElement(XmlTextReader xamlReader, XmlTextWriter htmlWriter, StringBuilder inlineStyle)
        {
            Debug.Assert(xamlReader.NodeType == XmlNodeType.Element);
            if (htmlWriter == null)
            {
                WriteElementContent(xamlReader, null, null);
            }
            else
            {
                string localName = null;
                switch (xamlReader.Name)
                {
                    case "Run":
                    case "Span":
                        localName = "SPAN";
                        break;

                    case "InlineUIContainer":
                        localName = "SPAN";
                        break;

                    case "Bold":
                        localName = "B";
                        break;

                    case "Italic":
                        localName = "I";
                        break;

                    case "Paragraph":
                        localName = "P";
                        break;

                    case "BlockUIContainer":
                        localName = "DIV";
                        break;

                    case "Section":
                        localName = "DIV";
                        break;

                    case "Table":
                        localName = "TABLE";
                        break;

                    case "TableColumn":
                        localName = "COL";
                        break;

                    case "TableRowGroup":
                        localName = "TBODY";
                        break;

                    case "TableRow":
                        localName = "TR";
                        break;

                    case "TableCell":
                        localName = "TD";
                        break;

                    case "List":
                    {
                        string attribute = xamlReader.GetAttribute("MarkerStyle");
                        if (((((attribute != null) && (attribute != "None")) && ((attribute != "Disc") && (attribute != "Circle"))) && (attribute != "Square")) && !(attribute == "Box"))
                        {
                            localName = "OL";
                            break;
                        }
                        localName = "UL";
                        break;
                    }
                    case "ListItem":
                        localName = "LI";
                        break;

                    default:
                        localName = null;
                        break;
                }
                if ((htmlWriter != null) && (localName != null))
                {
                    htmlWriter.WriteStartElement(localName);
                    WriteFormattingProperties(xamlReader, htmlWriter, inlineStyle);
                    WriteElementContent(xamlReader, htmlWriter, inlineStyle);
                    htmlWriter.WriteEndElement();
                }
                else
                {
                    WriteElementContent(xamlReader, null, null);
                }
            }
        }

        private static void WriteElementContent(XmlTextReader xamlReader, XmlTextWriter htmlWriter, StringBuilder inlineStyle)
        {
            Debug.Assert(xamlReader.NodeType == XmlNodeType.Element);
            bool flag = false;
            if (!xamlReader.IsEmptyElement)
            {
                while (ReadNextToken(xamlReader) && (xamlReader.NodeType != XmlNodeType.EndElement))
                {
                    switch (xamlReader.NodeType)
                    {
                        case XmlNodeType.Element:
                        {
                            if (!xamlReader.Name.Contains("."))
                            {
                                if (((htmlWriter != null) && !flag) && (inlineStyle.Length > 0))
                                {
                                    htmlWriter.WriteAttributeString("STYLE", inlineStyle.ToString());
                                    inlineStyle.Remove(0, inlineStyle.Length);
                                }
                                flag = true;
                                WriteElement(xamlReader, htmlWriter, inlineStyle);
                            }
                            else
                            {
                                AddComplexProperty(xamlReader, inlineStyle);
                            }
                            Debug.Assert((xamlReader.NodeType == XmlNodeType.EndElement) || ((xamlReader.NodeType == XmlNodeType.Element) && xamlReader.IsEmptyElement));
                            continue;
                        }
                        case XmlNodeType.Text:
                        case XmlNodeType.CDATA:
                        case XmlNodeType.SignificantWhitespace:
                            if (htmlWriter != null)
                            {
                                if (!(flag || (inlineStyle.Length <= 0)))
                                {
                                    htmlWriter.WriteAttributeString("STYLE", inlineStyle.ToString());
                                }
                                htmlWriter.WriteString(xamlReader.Value);
                            }
                            flag = true;
                            break;

                        case XmlNodeType.Comment:
                            if (htmlWriter != null)
                            {
                                if (!(flag || (inlineStyle.Length <= 0)))
                                {
                                    htmlWriter.WriteAttributeString("STYLE", inlineStyle.ToString());
                                }
                                htmlWriter.WriteComment(xamlReader.Value);
                            }
                            flag = true;
                            break;
                    }
                }
                Debug.Assert(xamlReader.NodeType == XmlNodeType.EndElement);
            }
            else
            {
                if (((htmlWriter != null) && !flag) && (inlineStyle.Length > 0))
                {
                    htmlWriter.WriteAttributeString("STYLE", inlineStyle.ToString());
                    inlineStyle.Remove(0, inlineStyle.Length);
                }
                flag = true;
            }
        }

        private static bool WriteFlowDocument(XmlTextReader xamlReader, XmlTextWriter htmlWriter)
        {
            if (!ReadNextToken(xamlReader))
            {
                return false;
            }
            if ((xamlReader.NodeType != XmlNodeType.Element) || (xamlReader.Name != "FlowDocument"))
            {
                return false;
            }
            StringBuilder inlineStyle = new StringBuilder();
            htmlWriter.WriteStartElement("HTML");
            htmlWriter.WriteStartElement("BODY");
            WriteFormattingProperties(xamlReader, htmlWriter, inlineStyle);
            WriteElementContent(xamlReader, htmlWriter, inlineStyle);
            htmlWriter.WriteEndElement();
            htmlWriter.WriteEndElement();
            return true;
        }

        private static void WriteFormattingProperties(XmlTextReader xamlReader, XmlTextWriter htmlWriter, StringBuilder inlineStyle)
        {
            Debug.Assert(xamlReader.NodeType == XmlNodeType.Element);
            inlineStyle.Remove(0, inlineStyle.Length);
            if (xamlReader.HasAttributes)
            {
                bool flag = false;
                while (xamlReader.MoveToNextAttribute())
                {
                    string str = null;
                    switch (xamlReader.Name)
                    {
                        case "Background":
                            str = "background-color:" + ParseXamlColor(xamlReader.Value) + ";";
                            break;

                        case "FontFamily":
                            str = "font-family:" + xamlReader.Value + ";";
                            break;

                        case "FontStyle":
                            str = "font-style:" + xamlReader.Value.ToLower() + ";";
                            break;

                        case "FontWeight":
                            str = "font-weight:" + xamlReader.Value.ToLower() + ";";
                            break;

                        case "FontSize":
                            str = "font-size:" + xamlReader.Value + ";";
                            break;

                        case "Foreground":
                            str = "color:" + ParseXamlColor(xamlReader.Value) + ";";
                            break;

                        case "TextDecorations":
                            str = "text-decoration:underline;";
                            break;

                        case "Padding":
                            str = "padding:" + ParseXamlThickness(xamlReader.Value) + ";";
                            break;

                        case "Margin":
                            str = "margin:" + ParseXamlThickness(xamlReader.Value) + ";";
                            break;

                        case "BorderThickness":
                            str = "border-width:" + ParseXamlThickness(xamlReader.Value) + ";";
                            flag = true;
                            break;

                        case "BorderBrush":
                            str = "border-color:" + ParseXamlColor(xamlReader.Value) + ";";
                            flag = true;
                            break;

                        case "TextIndent":
                            str = "text-indent:" + xamlReader.Value + ";";
                            break;

                        case "TextAlignment":
                            str = "text-align:" + xamlReader.Value + ";";
                            break;

                        case "Width":
                            str = "width:" + xamlReader.Value + ";";
                            break;

                        case "ColumnSpan":
                            htmlWriter.WriteAttributeString("COLSPAN", xamlReader.Value);
                            break;

                        case "RowSpan":
                            htmlWriter.WriteAttributeString("ROWSPAN", xamlReader.Value);
                            break;
                    }
                    if (str != null)
                    {
                        inlineStyle.Append(str);
                    }
                }
                if (flag)
                {
                    inlineStyle.Append("border-style:solid;mso-element:para-border-div;");
                }
                xamlReader.MoveToElement();
                Debug.Assert(xamlReader.NodeType == XmlNodeType.Element);
            }
        }
    }
}

