﻿namespace WpfRichText.Ex.XamlToHtmlParser
{
    using System;
    using System.Collections.Generic;
    using System.Text;
    using System.Xml;

    internal class HtmlParser
    {
        private XmlDocument _document = new XmlDocument();
        private HtmlLexicalAnalyzer _htmlLexicalAnalyzer;
        private Stack<XmlElement> _openedElements = new Stack<XmlElement>();
        private Stack<XmlElement> _pendingInlineElements = new Stack<XmlElement>();
        internal const string HtmlEndFragmentComment = "<!--EndFragment-->";
        internal const string HtmlHeader = "Version:1.0\r\nStartHTML:{0:D10}\r\nEndHTML:{1:D10}\r\nStartFragment:{2:D10}\r\nEndFragment:{3:D10}\r\nStartSelection:{4:D10}\r\nEndSelection:{5:D10}\r\n";
        internal const string HtmlStartFragmentComment = "<!--StartFragment-->";
        internal const string XhtmlNamespace = "http://www.w3.org/1999/xhtml";

        private HtmlParser(string inputString)
        {
            this._htmlLexicalAnalyzer = new HtmlLexicalAnalyzer(inputString);
            this._htmlLexicalAnalyzer.GetNextContentToken();
        }

        private void AddComment(string comment)
        {
            this.OpenPendingInlineElements();
            this.InvariantAssert(this._openedElements.Count > 0, "AddComment: Stack of opened elements cannot be empty, as we have at least one artificial root element");
            XmlElement element = this._openedElements.Peek();
            XmlComment newChild = this._document.CreateComment(comment);
            element.AppendChild(newChild);
        }

        private void AddEmptyElement(XmlElement htmlEmptyElement)
        {
            this.InvariantAssert(this._openedElements.Count > 0, "AddEmptyElement: Stack of opened elements cannot be empty, as we have at least one artificial root element");
            this._openedElements.Peek().AppendChild(htmlEmptyElement);
        }

        internal static string AddHtmlClipboardHeader(string htmlString)
        {
            StringBuilder builder = new StringBuilder();
            int num = "Version:1.0\r\nStartHTML:{0:D10}\r\nEndHTML:{1:D10}\r\nStartFragment:{2:D10}\r\nEndFragment:{3:D10}\r\nStartSelection:{4:D10}\r\nEndSelection:{5:D10}\r\n".Length + (6 * ("0123456789".Length - "{0:D10}".Length));
            int num2 = num + htmlString.Length;
            int index = htmlString.IndexOf("<!--StartFragment-->", 0);
            if (index >= 0)
            {
                index = (num + index) + "<!--StartFragment-->".Length;
            }
            else
            {
                index = num;
            }
            int num4 = htmlString.IndexOf("<!--EndFragment-->", 0);
            if (num4 >= 0)
            {
                num4 = num + num4;
            }
            else
            {
                num4 = num2;
            }
            builder.AppendFormat("Version:1.0\r\nStartHTML:{0:D10}\r\nEndHTML:{1:D10}\r\nStartFragment:{2:D10}\r\nEndFragment:{3:D10}\r\nStartSelection:{4:D10}\r\nEndSelection:{5:D10}\r\n", new object[] { num, num2, index, num4, index, num4 });
            builder.Append(htmlString);
            return builder.ToString();
        }

        private void AddTextContent(string textContent)
        {
            this.OpenPendingInlineElements();
            this.InvariantAssert(this._openedElements.Count > 0, "AddTextContent: Stack of opened elements cannot be empty, as we have at least one artificial root element");
            XmlElement element = this._openedElements.Peek();
            XmlText newChild = this._document.CreateTextNode(textContent);
            element.AppendChild(newChild);
        }

        private void CloseElement(string htmlElementName)
        {
            this.InvariantAssert(this._openedElements.Count > 0, "CloseElement: Stack of opened elements cannot be empty, as we have at least one artificial root element");
            if ((this._pendingInlineElements.Count > 0) && (this._pendingInlineElements.Peek().LocalName == htmlElementName))
            {
                XmlElement newChild = this._pendingInlineElements.Pop();
                this.InvariantAssert(this._openedElements.Count > 0, "CloseElement: Stack of opened elements cannot be empty, as we have at least one artificial root element");
                this._openedElements.Peek().AppendChild(newChild);
            }
            else if (this.IsElementOpened(htmlElementName))
            {
                while (this._openedElements.Count > 1)
                {
                    XmlElement htmlElement = this._openedElements.Pop();
                    if (htmlElement.LocalName == htmlElementName)
                    {
                        break;
                    }
                    if (HtmlSchema.IsInlineElement(htmlElement.LocalName))
                    {
                        this._pendingInlineElements.Push(this.CreateElementCopy(htmlElement));
                    }
                }
            }
        }

        private XmlElement CreateElementCopy(XmlElement htmlElement)
        {
            XmlElement element = this._document.CreateElement(htmlElement.LocalName, "http://www.w3.org/1999/xhtml");
            for (int i = 0; i < htmlElement.Attributes.Count; i++)
            {
                XmlAttribute attribute = htmlElement.Attributes[i];
                element.SetAttribute(attribute.Name, attribute.Value);
            }
            return element;
        }

        internal static string ExtractHtmlFromClipboardData(string htmlDataString)
        {
            int index = htmlDataString.IndexOf("StartHTML:");
            if (index < 0)
            {
                return "ERROR: Urecognized html header";
            }
            index = int.Parse(htmlDataString.Substring(index + "StartHTML:".Length, "0123456789".Length));
            if ((index < 0) || (index > htmlDataString.Length))
            {
                return "ERROR: Urecognized html header";
            }
            int length = htmlDataString.IndexOf("EndHTML:");
            if (length < 0)
            {
                return "ERROR: Urecognized html header";
            }
            length = int.Parse(htmlDataString.Substring(length + "EndHTML:".Length, "0123456789".Length));
            if (length > htmlDataString.Length)
            {
                length = htmlDataString.Length;
            }
            return htmlDataString.Substring(index, length - index);
        }

        private void InvariantAssert(bool condition, string message)
        {
            if (!condition)
            {
                throw new Exception("Assertion error: " + message);
            }
        }

        private bool IsElementOpened(string htmlElementName)
        {
            foreach (XmlElement element in this._openedElements)
            {
                if (element.LocalName == htmlElementName)
                {
                    return true;
                }
            }
            return false;
        }

        private void OpenInlineElement(XmlElement htmlInlineElement)
        {
            this._pendingInlineElements.Push(htmlInlineElement);
        }

        private void OpenPendingInlineElements()
        {
            if (this._pendingInlineElements.Count > 0)
            {
                XmlElement newChild = this._pendingInlineElements.Pop();
                this.OpenPendingInlineElements();
                this.InvariantAssert(this._openedElements.Count > 0, "OpenPendingInlineElements: Stack of opened elements cannot be empty, as we have at least one artificial root element");
                this._openedElements.Peek().AppendChild(newChild);
                this._openedElements.Push(newChild);
            }
        }

        private void OpenStructuringElement(XmlElement htmlElement)
        {
            if (HtmlSchema.IsBlockElement(htmlElement.LocalName))
            {
                while ((this._openedElements.Count > 0) && HtmlSchema.IsInlineElement(this._openedElements.Peek().LocalName))
                {
                    XmlElement element = this._openedElements.Pop();
                    this.InvariantAssert(this._openedElements.Count > 0, "OpenStructuringElement: stack of opened elements cannot become empty here");
                    this._pendingInlineElements.Push(this.CreateElementCopy(element));
                }
            }
            if (this._openedElements.Count > 0)
            {
                XmlElement element2 = this._openedElements.Peek();
                if (HtmlSchema.ClosesOnNextElementStart(element2.LocalName, htmlElement.LocalName))
                {
                    this._openedElements.Pop();
                    element2 = (this._openedElements.Count > 0) ? this._openedElements.Peek() : null;
                }
                if (element2 != null)
                {
                    element2.AppendChild(htmlElement);
                }
            }
            this._openedElements.Push(htmlElement);
        }

        private void ParseAttributes(XmlElement xmlElement)
        {
            while (((this._htmlLexicalAnalyzer.NextTokenType != HtmlTokenType.EOF) && (this._htmlLexicalAnalyzer.NextTokenType != HtmlTokenType.TagEnd)) && (this._htmlLexicalAnalyzer.NextTokenType != HtmlTokenType.EmptyTagEnd))
            {
                if (this._htmlLexicalAnalyzer.NextTokenType == HtmlTokenType.Name)
                {
                    string nextToken = this._htmlLexicalAnalyzer.NextToken;
                    this._htmlLexicalAnalyzer.GetNextEqualSignToken();
                    this._htmlLexicalAnalyzer.GetNextAtomToken();
                    string str2 = this._htmlLexicalAnalyzer.NextToken;
                    xmlElement.SetAttribute(nextToken, str2);
                }
                this._htmlLexicalAnalyzer.GetNextTagToken();
            }
        }

        internal static XmlElement ParseHtml(string htmlString)
        {
            HtmlParser parser = new HtmlParser(htmlString);
            return parser.ParseHtmlContent();
        }

        private XmlElement ParseHtmlContent()
        {
            XmlElement htmlElement = this._document.CreateElement("html", "http://www.w3.org/1999/xhtml");
            this.OpenStructuringElement(htmlElement);
            while (this._htmlLexicalAnalyzer.NextTokenType != HtmlTokenType.EOF)
            {
                string str;
                if (this._htmlLexicalAnalyzer.NextTokenType == HtmlTokenType.OpeningTagStart)
                {
                    this._htmlLexicalAnalyzer.GetNextTagToken();
                    if (this._htmlLexicalAnalyzer.NextTokenType == HtmlTokenType.Name)
                    {
                        str = this._htmlLexicalAnalyzer.NextToken.ToLower();
                        this._htmlLexicalAnalyzer.GetNextTagToken();
                        XmlElement xmlElement = this._document.CreateElement(str, "http://www.w3.org/1999/xhtml");
                        this.ParseAttributes(xmlElement);
                        if ((this._htmlLexicalAnalyzer.NextTokenType == HtmlTokenType.EmptyTagEnd) || HtmlSchema.IsEmptyElement(str))
                        {
                            this.AddEmptyElement(xmlElement);
                        }
                        else if (HtmlSchema.IsInlineElement(str))
                        {
                            this.OpenInlineElement(xmlElement);
                        }
                        else if (HtmlSchema.IsBlockElement(str) || HtmlSchema.IsKnownOpenableElement(str))
                        {
                            this.OpenStructuringElement(xmlElement);
                        }
                    }
                }
                else if (this._htmlLexicalAnalyzer.NextTokenType == HtmlTokenType.ClosingTagStart)
                {
                    this._htmlLexicalAnalyzer.GetNextTagToken();
                    if (this._htmlLexicalAnalyzer.NextTokenType == HtmlTokenType.Name)
                    {
                        str = this._htmlLexicalAnalyzer.NextToken.ToLower();
                        this._htmlLexicalAnalyzer.GetNextTagToken();
                        this.CloseElement(str);
                    }
                }
                else if (this._htmlLexicalAnalyzer.NextTokenType == HtmlTokenType.Text)
                {
                    this.AddTextContent(this._htmlLexicalAnalyzer.NextToken);
                }
                else if (this._htmlLexicalAnalyzer.NextTokenType == HtmlTokenType.Comment)
                {
                    this.AddComment(this._htmlLexicalAnalyzer.NextToken);
                }
                this._htmlLexicalAnalyzer.GetNextContentToken();
            }
            if (((htmlElement.FirstChild is XmlElement) && (htmlElement.FirstChild == htmlElement.LastChild)) && (htmlElement.FirstChild.LocalName.ToLower() == "html"))
            {
                htmlElement = (XmlElement) htmlElement.FirstChild;
            }
            return htmlElement;
        }
    }
}

